/* ***********************************************************
 * This file was automatically generated on 2012-07-13.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

using System;

namespace Tinkerforge
{
	/// <summary>
	///  Device for controlling a LCD with 4 lines a 20 characters
	/// </summary>
	public class BrickletLCD20x4 : Device 
	{
		private static byte FUNCTION_WRITE_LINE = 1;
		private static byte FUNCTION_CLEAR_DISPLAY = 2;
		private static byte FUNCTION_BACKLIGHT_ON = 3;
		private static byte FUNCTION_BACKLIGHT_OFF = 4;
		private static byte FUNCTION_IS_BACKLIGHT_ON = 5;
		private static byte FUNCTION_SET_CONFIG = 6;
		private static byte FUNCTION_GET_CONFIG = 7;
		private static byte FUNCTION_IS_BUTTON_PRESSED = 8;
		private static byte CALLBACK_BUTTON_PRESSED = 9;
		private static byte CALLBACK_BUTTON_RELEASED = 10;


		/// <summary>
		///  This callback is triggered when a button is pressed. The parameter is
		///  the number of the button (0 to 2).
		/// </summary>
		public delegate void ButtonPressed(byte button);

		/// <summary>
		///  This callback is triggered when a button is released. The parameter is
		///  the number of the button (0 to 2).
		/// </summary>
		public delegate void ButtonReleased(byte button);

		/// <summary>
		///  Creates an object with the unique device ID <c>uid</c>. This object can
		///  then be added to the IP connection.
		/// </summary>
		public BrickletLCD20x4(string uid) : base(uid) 
		{
			this.expectedName = "LCD 20x4 Bricklet";
			this.bindingVersion[0] = 1;
			this.bindingVersion[1] = 0;
			this.bindingVersion[2] = 0;
			messageCallbacks[CALLBACK_BUTTON_PRESSED] = new MessageCallback(CallbackButtonPressed);
			messageCallbacks[CALLBACK_BUTTON_RELEASED] = new MessageCallback(CallbackButtonReleased);
		}

		/// <summary>
		///  Writes text to a specific line (0 to 3) with a specific position 
		///  (0 to 19). The text can have a maximum of 20 characters.
		///  
		///  For example: (0, 7, "Hello") will write *Hello* in the middle of the
		///  first line of the display.
		///  
		///  The display uses a special charset that includes all ASCII characters except
		///  backslash and tilde. The LCD charset also includes several other non-ASCII characters, see
		///  the `charset specification &lt;https://github.com/Tinkerforge/lcd-20x4-bricklet/raw/master/datasheets/standard_charset.pdf&gt;`__
		///  for details. The Unicode example above shows how to specify non-ASCII characters
		///  and how to translate from Unicode to the LCD charset.
		/// </summary>
		public void WriteLine(byte line, byte position, string text)
		{
			byte[] data_ = new byte[26];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_WRITE_LINE, 1, data_);
			LEConverter.To((ushort)26, 2, data_);
			LEConverter.To(line, 4, data_);
			LEConverter.To(position, 5, data_);
			LEConverter.To(text, 6, 20, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Deletes all characters from the display.
		/// </summary>
		public void ClearDisplay()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_CLEAR_DISPLAY, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Turns the backlight on.
		/// </summary>
		public void BacklightOn()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_BACKLIGHT_ON, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Turns the backlight off.
		/// </summary>
		public void BacklightOff()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_BACKLIGHT_OFF, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns true if the backlight is on and false otherwise.
		/// </summary>
		public bool IsBacklightOn()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_IS_BACKLIGHT_ON, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_IS_BACKLIGHT_ON, out response);

			return LEConverter.BoolFrom(4, response);
		}

		/// <summary>
		///  Configures if the cursor (shown as "_") should be visible and if it
		///  should be blinking (shown as a blinking block). The cursor position
		///  is one character behind the the last text written with 
		///  <see cref="Tinkerforge.BrickletLCD20x4.WriteLine"/>.
		///  
		///  The default is (false, false).
		/// </summary>
		public void SetConfig(bool cursor, bool blinking)
		{
			byte[] data_ = new byte[6];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_CONFIG, 1, data_);
			LEConverter.To((ushort)6, 2, data_);
			LEConverter.To(cursor, 4, data_);
			LEConverter.To(blinking, 5, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the configuration as set by <see cref="Tinkerforge.BrickletLCD20x4.SetConfig"/>.
		/// </summary>
		public void GetConfig(out bool cursor, out bool blinking)
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CONFIG, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_CONFIG, out response);

			cursor = LEConverter.BoolFrom(4, response);
			blinking = LEConverter.BoolFrom(5, response);
		}

		/// <summary>
		///  Returns true if the button (0 to 2) is pressed. If you want to react
		///  on button presses and releases it is recommended to use the
		///  <see cref="Tinkerforge.BrickletLCD20x4.ButtonPressed"/> and <see cref="Tinkerforge.BrickletLCD20x4.ButtonReleased"/> callbacks.
		/// </summary>
		public bool IsButtonPressed(byte button)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_IS_BUTTON_PRESSED, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(button, 4, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_IS_BUTTON_PRESSED, out response);

			return LEConverter.BoolFrom(4, response);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void IsBacklightOn(out bool backlight)
		{
			backlight = IsBacklightOn();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void IsButtonPressed(byte button, out bool pressed)
		{
			pressed = IsButtonPressed(button);
		}

		internal int CallbackButtonPressed(byte[] data_)
		{
			byte button = LEConverter.ByteFrom(4, data_);

			((ButtonPressed)callbacks[CALLBACK_BUTTON_PRESSED])(button);
			return 5;
		}

		internal int CallbackButtonReleased(byte[] data_)
		{
			byte button = LEConverter.ByteFrom(4, data_);

			((ButtonReleased)callbacks[CALLBACK_BUTTON_RELEASED])(button);
			return 5;
		}

		/// <summary>
		///  Registers a callback function.
		/// </summary>
		public void RegisterCallback(System.Delegate d)
		{
			if(d.GetType() == typeof(ButtonPressed))
			{
				callbacks[CALLBACK_BUTTON_PRESSED] = d;
			}
			else if(d.GetType() == typeof(ButtonReleased))
			{
				callbacks[CALLBACK_BUTTON_RELEASED] = d;
			}
		}
	}
}
